# shellcheck shell=bash
# only allow this script to be sourced, by cf-support.sh, and not run directly
# as it relies on functions defined in cf-support.sh

# Here we use a bash quirk which allows return to be called outside of a function in a sourced file
# and a subshell to protect from returning from the script environment.
# If executed directly calling return will result in an error and return
# non-zero so proceed to the else near the end of this file..
if ! (return 0 2>/dev/null); then
  echo "run /var/cfengine/bin/cf-support instead of this file"
  exit 1
fi

# check if we are indeed on a hub
# this script should only be packaged in hub packages but just to be sure
# let's do some checks before-hand
if [ ! -d /var/cfengine/httpd ]; then
  echo "It seems that this host is not a CFEngine Enterprise Hub. Exiting."
  exit 1
fi

log_cmd "cf-serverd -Fv | awk '/=== BEGIN summary of access promises ===/,/=== END summary of access promises ===/'"
gzip_add /var/log/postgresql.log
# tmpdir defined in cf-support which sources this file so disable check
# shellcheck disable=SC2154
mkdir -p "$tmpdir$WORKDIR/httpd"
"$BINDIR"/rsync -az "$WORKDIR/httpd/logs" "$tmpdir$WORKDIR/httpd/"

file_add "$WORKDIR"/state/pg/data/postgresql.conf
file_add "$WORKDIR"/state/pg/data/pg_hba.conf

# Show 20 largest tables in cfdb database
# see https://www.postgresql.org/docs/current/catalog-pg-class.html
# and https://www.postgresql.org/docs/current/functions-admin.html (pg_total_relation_size function)
# Don't include TOAST tables as their size is included in pg_total_relation_size for each table
# and is not interesting to us directly: https://www.postgresql.org/docs/current/storage-toast.html.
# Don't include indexes (relkind <> 'i') as their size is included in pg_total_relation_size as well.
"$BINDIR/psql" cfdb -c "SELECT nspname || '.' || relname AS relation,
    pg_size_pretty(pg_total_relation_size(C.oid)) AS total_size
  FROM pg_class C
  LEFT JOIN pg_namespace N ON (N.oid = C.relnamespace)
  WHERE nspname NOT IN ('pg_catalog', 'information_schema')
  AND C.relkind <> 'i'
  AND nspname !~ '^pg_toast'
  ORDER BY pg_total_relation_size(C.oid) DESC
  Limit 20;" > "$tmpdir"/cfdb-biggest-tables.log
log_cmd "$BINDIR/psql cfdb -c \"SELECT count(*) as host_count from __hosts;\""
log_cmd "$BINDIR/psql cfdb -c \"select name, details, units, avg(value), count(value) from diagnostics group by name, details, units;\""
log_cmd "$BINDIR/psql cfdb -c \"select host, status, type, count(*) from __status group by host, status, type order by host, status;\""

# non_interactive defined in cf-support which sources this file so disable check
# shellcheck disable=SC2154
if [ "$non_interactive" -eq 0 ]; then
  read -r -p "Include masterfiles in support submission [Y/n]: " response
fi
response=${response:-y}
case $response in
  [yY][eE][sS]|[yY])
    mkdir -p "$tmpdir$WORKDIR"
    "$BINDIR/rsync" -az "$WORKDIR/masterfiles" "$tmpdir$WORKDIR"
    echo "Added directory $WORKDIR/masterfiles"
    ;;
  *)
    echo "Not including masterfiles in support submission"
    ;;
esac
